"use strict";

const express = require('express')
const bodyParser = require('body-parser')
const fileUpload = require('express-fileupload');
const redis = require('redis')
const morgan = require('morgan')
const mqtt = require('mqtt')
const client = mqtt.connect('mqtt://localhost')
const fs = require('fs')

const app = express()
app.use(morgan('combined'))
app.use(bodyParser.urlencoded({ extended: false }))
app.use(bodyParser.json())
app.use(fileUpload())
const node_id = "20sez54hq8"

const port = 8080

app.post('/serverless/deploy', (req, res) => {
    
    let runtime = req.body.runtime
    let file = req.files.serverless

    let functionHash = file.md5
    
    file.mv(__dirname + "/test/" + functionHash, function (err) {
        if (err) {
            console.log(err);
            res.send("error").status(400)
        }
        else {
            if (runtime === "container") {
                deployContainer('./test/', functionHash)
                    .then(() => {
                        res.json({
                            status: "success",
                            function_id: functionHash
                        })
                    })
                    .catch(err => {
                        res.json({
                            status: "error",
                            reason: err}).status(400)
                    })
            } else {
                res.json({
                    status: "success",
                    function_id: functionHash
                })
            }
        }
    })
    
})

function deployContainer(path, imageName) {
    return new Promise((resolve, reject) => {
        let buildStart = Date.now()
        
        fs.writeFile('./test/Dockerfile',
            `FROM node:latest
            WORKDIR /app
            COPY package.json /app
            RUN npm install
            COPY . /app
            
            CMD node ${imageName}`
            , function (err) {
                if (err) {
                    console.log("failed", err);
                    
                    reject(err);
                }
                else {
                    console.log('Dockerfile created');
                    const process = spawn('docker', ["build", "-t", imageName, path, "-q"]);

                    process.stdout.on('data', (data) => {
                        console.log(`stdout: ${data}`);

                    });

                    process.stderr.on('data', (data) => {
                        console.error(`stderr: ${data}`);
                    });

                    process.on('close', (code) => {
                        console.log(`child process exited with code ${code}`);
                        let timeDifference = Math.ceil((Date.now() - buildStart))
                        console.log("image build time taken: ", timeDifference);
                        resolve();
                    });
                }
            });
    })
}

app.post('/serverless/execute/:id', (req, res) => {

    let runtime = req.body.runtime
    let functionHash = req.params.id
    client.publish(node_id, JSON.stringify({
        "type": "execute",
        runtime, functionHash
    }))
    res.json({
        "status": "success"
    })
})



app.listen(port, () => console.log(`Server listening on port ${port}!`))